var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import simpleGit from 'simple-git';
import * as semver from 'semver';
import * as fs from 'fs-extra';
import * as path from 'path';
import { copy } from "../../util/copy";
import * as logger from "../../util/logger";
import { exec } from "./exec";
import { installDeps } from "./install-deps";
export function hasCache(repository) {
    return __awaiter(this, void 0, void 0, function* () {
        if (semver.valid(repository.version)) {
            const existed = yield fs.pathExists(repository.storage);
            if (existed)
                logger.info(`use the ${repository.name} template cache`);
            return existed;
        }
        return false;
    });
}
export function download(repository) {
    return __awaiter(this, void 0, void 0, function* () {
        yield fs.ensureDir(repository.storage);
        if (repository.type === 'git') {
            if (yield hasCache(repository))
                return repository.storage;
            logger.info(`clone ${repository.name} template from git...`);
            const git = simpleGit(repository.storage);
            const version = semver.valid(repository.version) ? `v${repository.version}` : repository.version;
            yield fs.emptyDir(repository.storage);
            yield git.clone(repository.name, repository.storage, ['--branch', version, '--single-branch']);
            yield installDeps(repository.storage);
            return repository.storage;
        }
        else if (repository.type === 'npm') {
            logger.info(`install ${repository.name} template from npm...`);
            const templatePath = path.join(repository.storage, 'node_modules', repository.name);
            if (hasCache(repository))
                templatePath;
            yield fs.emptyDir(repository.storage);
            yield fs.writeJSON(path.join(repository.storage, 'package.json'), {
                main: 'index.js',
                description: '',
                license: 'MIT',
            });
            const command = `npm install --production ${repository.name}@${repository.version}`;
            yield exec(command, { cwd: repository.storage });
            return templatePath;
        }
        else if (repository.type === 'fs') {
            logger.info(`copy ${repository.name} template from file system...`);
            yield copy(repository.name, repository.storage);
            yield installDeps(repository.storage);
            return repository.storage;
        }
        throw new Error(`Unable download the repository ${repository.name}`);
    });
}
