var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import * as fs from 'fs-extra';
import * as path from 'path';
import { readdeepdir } from "./readdeepdir";
import simpleGit from 'simple-git';
/**
 * Copy file from `src` to `dist`
 * Delete file in `dist` which `src` not existed
 */
export function syncDir(src, dist) {
    return __awaiter(this, void 0, void 0, function* () {
        yield fs.copy(src, dist);
        const git = simpleGit(dist);
        const files = yield readdeepdir(dist, {
            filter: (filepath) => __awaiter(this, void 0, void 0, function* () {
                const result = yield git.checkIgnore(filepath);
                return !result.length;
            }),
        });
        const promises = files.map((filename) => __awaiter(this, void 0, void 0, function* () {
            const filepath = path.join(src, filename);
            if (yield fs.pathExists(filepath))
                return;
            yield fs.remove(filepath);
        }));
        yield Promise.all(promises);
    });
}
