"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.diff = void 0;
const readdeepdir_1 = require("../util/readdeepdir");
const show_diff_1 = require("./show-diff");
const fs = require("fs-extra");
const path = require("path");
const chalk = require("chalk");
const simple_git_1 = require("simple-git");
const logger = require("../util/logger");
function diffModified(cwd, tmpDir, options) {
    return __awaiter(this, void 0, void 0, function* () {
        const filepaths = yield readdeepdir_1.readdeepdir(tmpDir);
        const git = simple_git_1.default(cwd);
        function isIgnore(filepath) {
            return __awaiter(this, void 0, void 0, function* () {
                const result = yield git.checkIgnore(filepath);
                return Boolean(result.length);
            });
        }
        const errors = [];
        for (const filepath of filepaths) {
            const srcfilepath = path.join(cwd, filepath);
            const existed = (yield fs.pathExists(srcfilepath)) && !(yield isIgnore(srcfilepath));
            const oldBuffer = existed ? yield fs.readFile(path.join(cwd, filepath)) : Buffer.from('');
            const newBuffer = yield fs.readFile(path.join(tmpDir, filepath));
            if (oldBuffer.equals(newBuffer))
                continue;
            if (options.showDiff) {
                errors.push(show_diff_1.showDiff(filepath, oldBuffer, newBuffer, options));
            }
            else {
                errors.push(chalk.yellow(`${filepath}: Not Match Template`));
            }
        }
        logger.error(errors.join('\n'));
    });
}
function diffRemoved(cwd, tmpDir, options) {
    return __awaiter(this, void 0, void 0, function* () {
        const git = simple_git_1.default(cwd);
        const files = yield readdeepdir_1.readdeepdir(cwd, {
            filter: (filepath) => __awaiter(this, void 0, void 0, function* () {
                const result = yield git.checkIgnore(filepath);
                return !result.length;
            }),
        });
        const errors = [];
        for (const filename of files) {
            const filepath = path.join(tmpDir, filename);
            if (yield fs.pathExists(filepath))
                return;
            if (options.showDiff) {
                errors.push(show_diff_1.showRemoved(filename));
            }
            else {
                errors.push(chalk.red(`${filepath}: Should be remove`));
            }
        }
        logger.error(errors.join('\n'));
    });
}
function diff(cwd, tmpDir, options) {
    return __awaiter(this, void 0, void 0, function* () {
        yield diffModified(cwd, tmpDir, options);
        yield diffRemoved(cwd, tmpDir, options);
    });
}
exports.diff = diff;
