"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.render = void 0;
const ajv_1 = require("ajv");
const R = require("ramda");
const path = require("path");
const parse_template_1 = require("../util/parse-template");
const logger = require("../util/logger");
const download_1 = require("./download");
const load_config_1 = require("./load-config");
const inquire_1 = require("./inquire");
const exec_loader_1 = require("./exec-loader");
const compile_1 = require("./compile");
const ajv = new ajv_1.default();
function render(cwd, repository, answers) {
    return __awaiter(this, void 0, void 0, function* () {
        const templatePath = yield download_1.download(repository);
        const config = yield load_config_1.loadConfig(templatePath);
        answers = yield inquire_1.inquire(config.questions, answers);
        logger.info('compile template...');
        for (const extension of config.extends) {
            if (extension.when) {
                const validate = ajv.compile(extension.when);
                const valid = validate(answers[repository.name]);
                if (!valid)
                    continue;
            }
            const subRepository = parse_template_1.parseTemplate(extension.template, extension.version);
            let subAnwsers = R.clone(answers);
            if (extension.answers)
                subAnwsers = Object.assign(Object.assign({}, subAnwsers), extension.answers);
            render(cwd, subRepository, subAnwsers);
        }
        const resource = new Map();
        for (const loader of config.loaders) {
            const result = yield exec_loader_1.execLoader(cwd, loader);
            for (const [key, value] of Object.entries(result)) {
                if (resource.has(key))
                    throw new Error(`Key Confilct - Loader:${loader.name}, key:${key}`);
                resource.set(key, value);
            }
        }
        resource.set('answers', answers);
        resource.set('milirc', {
            template: repository.name,
            version: config.version,
        });
        yield compile_1.compile(cwd, path.join(templatePath, 'templates'), config.templates, resource);
    });
}
exports.render = render;
