"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.update = void 0;
const fs = require("fs-extra");
const migrate_1 = require("./migrate");
const render_1 = require("./render");
const check_1 = require("./util/check");
const copy_1 = require("./util/copy");
const create_tmp_dir_1 = require("./util/create-tmp-dir");
const parse_template_1 = require("./util/parse-template");
const sync_dir_1 = require("./util/sync-dir");
function update(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const { template, version = 'latest', cwd = process.cwd(), force = false, answers = {}, } = options;
        if (!force)
            yield check_1.check(cwd);
        const tmpDir = yield create_tmp_dir_1.createTmpDir();
        yield copy_1.copy(cwd, tmpDir);
        const repository = parse_template_1.parseTemplate(template, version);
        yield render_1.render(tmpDir, repository, answers);
        yield migrate_1.migrate(tmpDir, repository, version);
        yield sync_dir_1.syncDir(tmpDir, cwd);
        yield fs.remove(tmpDir);
    });
}
exports.update = update;
